# ansible

![ansible](images/ansible.png "ansible")<!-- .element width="30%" -->


## ansible

* outil de
  * provisioning
  * gestion de config
  * déploiemenet d'application

* racheté par RedHat en octobre 2015

* outils équivalents
  * puppet, chief, salt ...


## ansible Φ

* idempotence
* héritage
* réutilisabilité
* parallélisation


## ansible

* écrit en python
  * python 2 par défaut
  * marche bien en python 3
    * virtualenv

* [ansible docs](https://docs.ansible.com/)


## ansible

* prérequis
  * sur la machine pilote (mgmt node)
    * ansible
  * sur le(s) noeud(s)
    * une connextion ssh ou PowerShell
    * python


## ansible

[![gestion ansible](images/ansible_mgmt.png "gestion ansible")](https://medium.com/formcept/configuration-management-and-continuous-deployment-cd0892dce998)


## terminologie

* **mgmt node** (machine pilote) machine surlaquelle ansible est installé et responsable de la configuration de toutes les machines gérées.
* **inventory** (inventaire) fichier contenant les ip de tous les serveurs à configurer
* **playbook** gère la configuration à déployer sur chaque serveur
* **task** fichier où sont définies les actions réalisées par le playbook


## terminologie

* **module** actions plus ou moins complexe utilisables à partir des **tasks**. ansible possède de [nombreux modules natifs](https://docs.ansible.com/ansible/latest/modules/modules_by_category.html). Il est possible d'écrire ses propres modules.
* **role** permet d'organiser les playbooks en parties claires et réutilisables


## terminologie

* **play** est l'exécution d'un playbook sur un serveur
* **facts** information collectée par ansible sur le système du serveur à configurer
* **handlers** similaire aux **tasks** mais appelable à partir d'une **task** (redémarrage de service par exemple)


## inventory

* fichier texte au format *ini*
* organiser par groupes
* [inventory docs](https://docs.ansible.com/ansible/latest/user_guide/intro_inventory.html)

```ini
[ovh]
duncan
ispconfig-duncan
gitlab-runner-duncan

[all:vars]
environment                = production
ansible_python_interpreter = /usr/bin/python3
ansible_user               = limosadm
```


## tips ssh

* utilisez ~/.ssh/config

```
Host duncan
  User limosadm
  IdentityFile ~/.ssh/keys/limosadm
  Hostname duncan.isima.fr

Host ispconfig-duncan
  User limosadm
  Hostname 10.10.100.2
  IdentityFile ~/.ssh/keys/limosadm
  ProxyCommand ssh duncan -W %h:%p
```


## ansible / ad-hoc command

```
$ ansible all --inventory-file=inventory.ini --module-name ping
$ ansible ovh -a "/usr/bin/uptime"

duncan | CHANGED | rc=0 >>
 18:15:42 up 16 days, 43 min,  1 user,  load average: 0,10, 0,10, 0,03

gitlab-runner-duncan | CHANGED | rc=0 >>
 18:15:45 up 15 days,  1:00,  1 user,  load average: 0.06, 0.03, 0.01

ispconfig-duncan | CHANGED | rc=0 >>
 18:15:48 up 15 days,  1:00,  1 user,  load average: 0.02, 0.02, 0.00
```


## ansible / ad-hoc command

* [docs ad-hoc command](https://docs.ansible.com/ansible/latest/user_guide/intro_adhoc.html)
  * shell
  * packages
  * users / groups
  * déploiement (git)
  * services


## playbook

```
ansible-playbook ansible/playbooks/ovh/duncan.yml
```

* `--check` simule les tâches à effectuer sans les effectuer (dry-run)
* `--diff` indique ce qui change
* `-v`, `-vv`, `-vvv`, `-vvvv` pour la verbosité
* `--list-tags` liste tous les tags disponibles dans le playbook
* `--tags debug` n'exécute que les tâches du playbook ayant un tag `debug`
* `--skip-tags` exécute toutes les tâches du playbook sauf celles ayant un  tag `debug`
* `always` tag spécial exécuté à tous les coups

[docs playbooks](https://docs.ansible.com/ansible/latest/user_guide/playbooks.html)


## variable

* pas de `-` pas de `.` pas de nuémrique pure dans les noms de variables
* via la ligne de commande
* dans les rôles
  * default
* dans l'inventaire
* `group_vars`, `host_vars`
* extra vars (gagne toujours à la précédence)
* [héritage des variables](https://docs.ansible.com/ansible/latest/user_guide/playbooks_variables.html#variable-precedence-where-should-i-put-a-variable)


## variable & secret

* from env ou Vault
* acces dynamique
* `ansible-vault` nous on utilise vault by HashiCorp


## variable & secret

* [var_prompts](https://docs.ansible.com/ansible/latest/user_guide/playbooks_prompts.html)
  * ceux là devraient être dans un keepass
    * gitlab
    * vault


## task / basic

* [debug](https://docs.ansible.com/ansible/latest/modules/debug_module.html)
* [raw](https://docs.ansible.com/ansible/latest/modules/raw_module.html)
* [command](https://docs.ansible.com/ansible/latest/modules/command_module.html#command-module)
* [shell](https://docs.ansible.com/ansible/latest/modules/shell_module.html)


## task / basic

* register
* loop
* set_fact
* pre_task
* when
* tags


## modules

* file
* infile
* copy
* template
* package
* stat
* get_url
* unarchive
* user
* systemd
* pip
* pexpect (require pexcpect)

## roles


## remote roles


### Récupérer les roles nécessaires

* mettre à jour `requirements.yml` avec les rôles nécessaires

* récupérer les rôles

```
ansible-galaxy install -f -r requirements.yml -p ansible/roles/public
```


## plugins

* callback
* filter
* lookup
* module_utils


## skeleton

* https://github.com/mrjk/ansible-skel

```
ansible-galaxy init --role-skeleton /path/to/stack/ansible/roles/skeletons/role-with-vagrant gitlab
```

## ansible.cfg
